@@ -15,7 +15,11 @@ from metaphor.common.utils import start_of_day
 from metaphor.snowflake.accessed_object import AccessedObject
 from metaphor.snowflake.auth import connect
 from metaphor.snowflake.usage.config import SnowflakeUsageRunConfig
-from metaphor.snowflake.utils import QueryWithParam, async_execute
+from metaphor.snowflake.utils import (
+    QueryWithParam,
+    async_execute,
+    fetch_query_history_count,
+)
 
 logger = get_logger(__name__)
 
@@ -73,28 +77,16 @@ class SnowflakeUsageExtractor(BaseExtractor):
         conn = connect(config)
 
         with conn:
-            cursor = conn.cursor()
-
-            cursor.execute(
-                f"""
-                SELECT COUNT(1)
-                FROM SNOWFLAKE.ACCOUNT_USAGE.QUERY_HISTORY
-                WHERE EXECUTION_STATUS = 'SUCCESS' and START_TIME > %s
-                  {excluded_usernames_clause}
-                """,
-                (
-                    start_date,
-                    *config.excluded_usernames,
-                ),
+            count = fetch_query_history_count(
+                conn, start_date, config.excluded_usernames
             )
-            count = cursor.fetchone()[0]
             batches = math.ceil(count / self.batch_size)
             logger.info(f"Total {count} queries, dividing into {batches} batches")
 
             queries = {
                 x: QueryWithParam(
                     f"""
-                    SELECT START_TIME, DIRECT_OBJECTS_ACCESSED
+                    SELECT START_TIME, q.USER_NAME, DIRECT_OBJECTS_ACCESSED
                     FROM SNOWFLAKE.ACCOUNT_USAGE.QUERY_HISTORY q
                     JOIN SNOWFLAKE.ACCOUNT_USAGE.ACCESS_HISTORY a
                       ON a.QUERY_ID = q.QUERY_ID
@@ -130,10 +122,12 @@ class SnowflakeUsageExtractor(BaseExtractor):
 
     def _parse_access_logs(self, batch_number: str, access_logs: List[Tuple]) -> None:
         logger.info(f"access logs batch #{batch_number}")
-        for start_time, accessed_objects in access_logs:
-            self._parse_access_log(start_time, accessed_objects)
+        for start_time, username, accessed_objects in access_logs:
+            self._parse_access_log(start_time, username, accessed_objects)
 
-    def _parse_access_log(self, start_time: datetime, accessed_objects: str) -> None:
+    def _parse_access_log(
+        self, start_time: datetime, username: str, accessed_objects: str
+    ) -> None:
         try:
             objects = parse_raw_as(List[AccessedObject], accessed_objects)
             for obj in objects:
@@ -170,12 +164,15 @@ class SnowflakeUsageExtractor(BaseExtractor):
 
                 if self._use_history:
                     UsageUtil.update_table_and_columns_usage_history(
-                        self._datasets[table_name].usage_history, columns
+                        self._datasets[table_name].usage_history, columns, username
                     )
                 else:
                     UsageUtil.update_table_and_columns_usage(
-                        self._datasets[table_name].usage, columns, start_time, utc_now
+                        self._datasets[table_name].usage,
+                        columns,
+                        start_time,
+                        utc_now,
+                        username,
                     )
-        except Exception as e:
-            logger.error(f"access log error, objects: {accessed_objects}")
-            logger.exception(e)
+        except Exception:
+            logger.exception(f"access log error, objects: {accessed_objects}")
